%
% features = decdeltarot(input, neigh)
%
% decode relative rotation encoding
%
% input       - model in relative rotation encoding
% neigh       - Nx3 matrix with indices of neighbouring triangles
% features    - model in translation invariant encoding
%
function features = decrelrot(input, neigh)

if size(input,2) == 1
    input = input';
end

step = 15;
N = size(neigh, 1);

direction = [0;0;0];

landmarks = 1;
weight = ones(size(landmarks));

features = zeros(1, 10*N);

A = sparse(4*N,3*N);
row = 1;

for i = 1:N
    n = neigh(i,1);
    if n > i
        delta = fromrotvec(input((i-1)*step+1:(i-1)*step+3)');
        A(row:row+2,(n-1)*3+1:(n-1)*3+3) = -eye(3);
        A(row:row+2,(i-1)*3+1:(i-1)*3+3) = delta';
        row = row + 3;
    end

    n = neigh(i,2);
    if n > i
        delta = fromrotvec(input((i-1)*step+4:(i-1)*step+6)');
        A(row:row+2,(n-1)*3+1:(n-1)*3+3) = -eye(3);
        A(row:row+2,(i-1)*3+1:(i-1)*3+3) = delta';
        row = row + 3;
    end

    n = neigh(i,3);
    if n > i
        delta = fromrotvec(input((i-1)*step+7:(i-1)*step+9)');
        A(row:row+2,(n-1)*3+1:(n-1)*3+3) = -eye(3);
        A(row:row+2,(i-1)*3+1:(i-1)*3+3) = delta';
        row = row + 3;
    end
    features((i-1)*10+5:(i-1)*10+10) = input((i-1)*step+10:(i-1)*step+15);
end

rhs = zeros(size(A,1),3);

for i = 1:length(landmarks)
    A(row:row+2,(landmarks(i)-1)*3+1:(landmarks(i)-1)*3+3) = eye(3)*weight(i);
    rhs(row:row+2,1:3) = fromrotvec(direction(i,:)')'*weight(i);
    row = row + 3;
end

At = A';
Rs = (At*A) \ (At*rhs);

for i = 0:N-1
    [U, S, V] = svd(Rs(i*3+1:i*3+3,:)');
    x = U * V';
    features(i*10+1:i*10+3) = torotvec(x);
end
