%
% model = reconstructmodel(features, faces, template)
%
% convert features into transformation matrices and deform template with them
% then reconstruct the model in euclidean coordinates
%
% features - model in translation invariant encoding
% faces    - Nx3 matrix of faces i.e. vertex indices
% template - vertex positions of template model
%
function model = dectrans(features, faces, template)

ftrs = features;

weight = 1;

fixvertex = 1;
fixto = zeros(1,3);

M = spalloc(size(template,1), size(template,1), 10*size(template,1));
dx = zeros(size(template,1), 3);

% build laplacian system
for i = 1:size(faces,1)
    v = faces(i,:);
    at = (i-1)*10+1;
    % reconstruct transformation matrix
    R = fromrotvec(ftrs(at:at+2)');
    S = zeros(3,3);
    S(1,:) = ftrs(at+4:at+6);
    S(:,1) = ftrs(at+4:at+6);
    S(2,2:3) = ftrs(at+7:at+8);
    S(2:3,2) = ftrs(at+7:at+8);
    S(3,3) = ftrs(at+9);
    T = R * S;

    % transform triangle
    x = T * [template(v(1),:)',template(v(2),:)',template(v(3),:)'];

    p12 = x(:,1) - x(:,2);
    p13 = x(:,1) - x(:,3);
    p23 = x(:,2) - x(:,3);

    p12 = p12 / norm(p12);
    p13 = p13 / norm(p13);
    p23 = p23 / norm(p23);

    wts = [p13'*p23 -p12'*p23 p12'*p13];
    wij3 = [0 wts(1) / sqrt(1-wts(1)^2)  wts(2) / sqrt(1-wts(2)^2)
            0 0  wts(3) / sqrt(1-wts(3)^2)
            0 0 0];
    wij3 = wij3 + wij3';
    WIJ = wij3-diag(sum(wij3));
    M(v, v) = M(v, v) + WIJ;
    dx(v,:) = dx(v,:) + WIJ * x';
end

weight = ones(length(fixvertex), 1) * weight;

for i = 1:length(fixvertex)
    M(size(template,1)+i,fixvertex(i)) = weight(i);
end
dx(size(template,1)+1:size(template,1)+length(fixvertex),:) = fixto .* repmat(weight, 1, 3);

M = real(M);
dx = real(dx);
Mt = M';
model = (Mt*M) \ (Mt*dx);
