%
% [vertices, faces, normals] = readobj(name)
%
% read vertices, faces, and normals from OBJ file
%
function [vertices, faces, normals, uvcoords] = readobj(name)

vertices = [];
faces = [];
normals = [];
uvcoords = [];

matname = regexprep(name, 'obj', 'mat');
fid = fopen(matname, 'r');
if fid > 0
    fclose(fid);
    vertices = load(matname,'-ascii');
    return
end

fid = fopen(name, 'r');
if fid < 0
    disp('error: file could not be opened');
    return
end

at = 1;
atn = 1;
atf = 1;
atuv = 1;
while feof(fid) == 0
    tline = fgetl(fid);
    if tline == -1
        break
    end
    if ~isempty(strfind(tline, 'v '))
        vertices(at,1:3) = sscanf(tline, 'v %f %f %f')';
        at = at + 1;
    elseif ~isempty(strfind(tline, 'vt '))
        uvcoords(atuv,1:2) = sscanf(tline, 'vt %f %f')';
        atuv = atuv + 1;
    elseif ~isempty(strfind(tline, 'vn '))
        normals(atn,1:3) = sscanf(tline, 'vn %f %f %f')';
        atn = atn + 1;
    elseif ~isempty(strfind(tline, 'f '))
        x = sscanf(tline, 'f %d//%d %d//%d %d//%d')';
        if length(x) == 6
            faces(atf,1:3) = x(1:2:6);
        else
            x = sscanf(tline, 'f %d %d %d %d')';
            if length(x) == 4
                faces(atf,1:3) = x(1:3);
                atf = atf + 1;
                faces(atf,1:3) = [x(1), x(3), x(4)];
            else
                x = sscanf(tline, 'f %d %d %d')';
                if length(x) == 3
                    faces(atf,1:3) = x;
                end
            end
        end
        atf = atf + 1;
    end
end
fclose(fid);
